#include "latfold.h"

static gint calc_score_partially(Atom *atom, gint katom);
static void copy_atom(Atom *src, Atom *dst, gint natom);
static void reset_atom_move(LatFoldData *data);
static gboolean try_bend(LatFoldData *data, gint origin, gint direction, gint rot);
static void rotate(XY *s, XY *o, gint rot, XY *d);

/** @brief 分子を初期状態に配置する関数
 *  @param[in] data LatFoldDataのポインター
 *  @param[in] seed 乱数の種
 */
void make_molecule(LatFoldData *data, gint seed) {
	GRand *rand;
	AtomType prev_type = ATOM_DUMMY;
	gint i;
	/* 直線の分子を(-L/2, L/2)となるように作る */
	rand = g_rand_new_with_seed(seed);
	for (i = 0; i < data->natom; ++i) {
		data->atom[i].x = i - data->natom / 2;
		data->atom[i].y = 0;
		if (i == 0) {
			data->atom[0].type = (AtomType)g_rand_int_range(rand, 1, 3);
		} else if (i == 1 || i == data->natom - 1) {
			data->atom[i].type = data->atom[i-1].type;
		} else if (data->atom[i-2].type == data->atom[i-1].type) {
			data->atom[i].type = (AtomType)g_rand_int_range(rand, 1, 3); } else {
			data->atom[i].type = data->atom[i-1].type;
		}
		if (data->atom[i].type == ATOM_H) {
			++(data->nhatom);
		} else {
			++(data->npatom);
		}
		data->work[i].type = data->atom[i].type;
		if (prev_type == data->atom[i].type && prev_type == ATOM_H) {
			++data->score0;
		} else {
			prev_type = data->atom[i].type;
		}
	}
	return;
}

/*! @brief LatFoldDataの初期化を行う。
 *  @param[in] gui LatFoldGUIのポインター
 *  @param[in] natom 原子の数
 *  @param[in] seed 乱数の種
 *  @return 新たに作成されたLatFoldData変数のポインタ値。
 *  不要になったら、latfold_data_destroy()を行わなければならない。
 */
LatFoldData *latfold_data_new(LatFoldGUI *gui, gint natom, gint seed) {
	LatFoldData *data;
	print_enter();
	data = g_malloc(sizeof(LatFoldData));
	data->natom = natom;
	data->npatom = 0;
	data->nhatom = 0;
	data->seed = seed;
	data->atom = g_malloc(sizeof(Atom)*data->natom);
	data->work = g_malloc(sizeof(Atom)*data->natom);
	data->move = g_malloc(sizeof(gboolean)*data->natom);
	data->ca = g_malloc(sizeof(CandidateAtom)*6);
	data->nca = 0;
	data->mode_preview = FALSE;
	data->score0 = 0;
	data->user_best = 0;
	data->solve = NULL;
	data->op_type = NONE;
	data->user_timer = g_timer_new();
	/* 分子を配置する */
	make_molecule(data, seed);
	/* GUIに関する部分を初期化する */
	latfold_gui_clist_clear(gui);
	set_npatom(gui, data->npatom);
	set_nhatom(gui, data->nhatom);
	gui->operatable = TRUE;
	print_leave();
	return data;
}

/** @brief 全得点を計算する
 *  @param[in] data LatFoldData型のポインター
 *  @param[in] atom Atom型配列
 *  @return 全得点
 *  @details 重複計算を避けるため、自分(0,0)から見て(1,0)と(0,1)のみを調べる
 */
gint calc_score(LatFoldData *data, Atom *atom) {
	return calc_score_partially(atom, data->natom);
}

/** @brief 部分得点を計算する
 *  @param[in] atom Atom型配列
 *  @param[in] katom 計算対象の原子の数
 *  @return 得点
 *  @details 重複計算を避けるため、自分(0,0)から見て(1,0)と(0,1)のみを調べる
 */
static gint calc_score_partially(Atom *atom, gint katom) {
	gint score;
	gint i, j, dx, dy, ix, iy;
	AtomType prev_type = ATOM_DUMMY;
	XY neighbor;
	print_enter();
	score = 0;
	for (i = 0; i < katom; ++i) {
		if (atom[i].type == ATOM_H && prev_type == ATOM_H) {
			--score;
		}
		prev_type = atom[i].type;
		if (atom[i].type != ATOM_H) {
			continue;
		}
		ix = atom[i].x;
		iy = atom[i].y;
		for (dx = 0; dx < 2; ++dx) {
			dy = 1 - dx;
			neighbor.x = ix + dx;
			neighbor.y = iy + dy;
			if ((j = lattice2atom(atom, katom, &neighbor)) >= 0
				&& atom[j].type == ATOM_H) {
				++score;
			}
		}
	}
	print_leave();
	return score;
}

/*! @brief LatFoldData変数を解放する。
 *  @param[in] data 解放したいLatFoldData変数
 */
void latfold_data_destroy(LatFoldData *data) {
	print_enter();
	if (data != NULL) {
		g_timer_destroy(data->user_timer);
		data->user_timer = NULL;
		g_nullify_pointer((gpointer)&data->atom);
		g_nullify_pointer((gpointer)&data->work);
		g_nullify_pointer((gpointer)&data->move);
		g_nullify_pointer((gpointer)&data->ca);
	}
	g_nullify_pointer((gpointer)&data);
	print_leave();
	return;
}

/** @brief 格子点にある原子が何番の原子か調べる関数
 *  @param[in] atom Atom型配列
 *  @param[in] natom 全原子の数
 *  @param[in] l lattice座標
 *  @retval 原子の番号 存在するとき
 *  @retval -1 存在しないとき
 */
gint lattice2atom(Atom *atom, gint natom, XY *l) {
	gint i;
	print_enter();
	for (i = 0; i < natom; ++i) {
		if (l->x == atom[i].x && l->y == atom[i].y) {
			return i;
		}
	}
	print_leave();
	return -1;
}

/** @brief 格子点にある原子が何番の原子か調べる関数
 *  @param[in] atom Atom型配列
 *  @param[in] natom 全原子の数
 *  @param[in] l1 1番目のlattice座標
 *  @param[in] l2 2番目のlattice座標
 *  @retval 原子の若い番号 存在するとき
 *  @retval -1 存在しないとき
 */
gint lattice2bond(Atom *atom, gint natom, XY *l1, XY *l2) {
	gint i;
	print_enter();
	for (i = 0; i < natom-1; ++i) {
		if (l1->x == atom[i].x && l1->y == atom[i].y 
				&& l2->x == atom[i+1].x && l2->y == atom[i+1].y) {
			return i;
		} else if (l2->x == atom[i].x && l2->y == atom[i].y 
				&& l1->x == atom[i+1].x && l1->y == atom[i+1].y) {
			return i;
		}
	}
	print_leave();
	return -1;
}

/** @brief 移動可能な場所かどうかを調べる関数
 *  @param[in] ca CandidateAtom型配列
 *  @param[in] nca 移動可能な場所
 *  @param[in] mouse Mouse型ポインター
 *  @retval 候補の番号 存在するとき
 *  @retval -1 存在しないとき
 */
gint candidate_place(CandidateAtom *ca, gint nca, Mouse *mouse) {
	XY *l1 = &(mouse->a1);
	XY *l2 = &(mouse->a2);
	gint i;
	print_enter();
	if (mouse->mtype == MOUSE_TYPE_ATOM) {
		for (i =0; i < nca; ++i) {
			if (l1->x == ca[i].x && l1->y == ca[i].y) {
				return i;
			}
		}
	} else if (mouse->mtype == MOUSE_TYPE_BOND) {
		for (i =0; i < nca; ++i) {
			if (l1->x == ca[i].x && l1->y == ca[i].y) {
				return i;
			} else if (l2->x == ca[i].x && l2->y == ca[i].y) {
				return i;
			}
		}
	}
	print_leave();
	return -1;
}

/** @brief Atom配列の原子をsrcからdstにコピーする関数
 *  @param[in] src コピー元のAtom配列
 *  @param[out] dst コピー先のAtom配列
 *  @param[in] natom 原子の数
 */
static void copy_atom(Atom *src, Atom *dst, gint natom) {
	gint i;
	print_enter();
	for (i = 0; i < natom; ++i) {
		dst[i].x = src[i].x;
		dst[i].y = src[i].y;
	}
	print_leave();
	return;
}

/** @brief bend後の原子の座標を求める関数
 *  @param[in] data LatFoldDataのポインター
 *  @param[in] origin bendの軸となる原子の番号
 *  @param[in] direction 軸から曲がる原子番号の方向
 *  @param[in] rot 回転角(90度単位)
 *  @retval TRUE bendに成功したとき
 *  @retval FALSE bendに失敗したとき
 */
static gboolean try_bend(LatFoldData *data, gint origin, gint direction, gint rot) {
	XY *oxy;
	gint i, j, start, end, rest_start, rest_end;
	gint ix, iy, jx, jy;
	print_enter();
	copy_atom(data->atom, data->work, data->natom);
	reset_atom_move(data);  /* 何度もこの関数が呼ばれるのでリセットが必要  */
	oxy = (XY *)&data->atom[origin];
	if (direction > 0) {
		rest_start = 0;
		rest_end = origin;
		start = origin + 1;
		end = data->natom - 1;
	} else {
		start = 0;
		end = origin - 1;
		rest_start = origin;
		rest_end = data->natom - 1;
	}
	for (i = start; i <= end; ++i) {
		XY *ixy;
		ixy = (XY *)&data->atom[i];
		rotate(ixy, oxy, rot, (XY *)&(data->work[i]));
	}
	for (i = start; i <= end; ++i) {
		data->move[i] = TRUE;
	}
	for (i = start; i <= end; ++i) {
		ix = data->work[i].x;
		iy = data->work[i].y;
		for (j = rest_start; j <= rest_end; ++j) {
			jx = data->atom[j].x;
			jy = data->atom[j].y;
			if (ix == jx && iy == jy) {
				return FALSE;
			}
		}
	}
	print_leave();
	return TRUE;
}

/** @brief 移動可能な移動先の格子点を調べる関数
 *  @param[in] data LatFoldDataのポインター
 *  @param[in] mouse クリックに関するマウス情報
 *  @retval nca 移動可能な格子点の数
 */
gint set_candidate_atom(LatFoldData *data, Mouse *mouse) {
	XY *ixy, jxy;
	gint nca = 0;
	gint neighbor, rot;
	gint iatom, i;
	print_enter();
	iatom = lattice2atom(data->atom, data->natom, &mouse->a1);
	if (iatom < 0) {
		data->nca = 0;
		return 0;
	}
	for (i = -1; i < 2 ; i += 2) {
		neighbor = iatom - i;
		if (neighbor >= 0 && neighbor < data->natom) {
			ixy = (XY *)&data->atom[neighbor];
			for (rot = 1; rot < 4; ++rot) {
				rotate(&mouse->a1, ixy, rot, &jxy);
				if (try_bend(data, neighbor, i, rot)) {
					data->ca[nca].x = jxy.x;
					data->ca[nca].y = jxy.y;
					data->ca[nca].origin = neighbor;
					data->ca[nca].direction = i;
					data->ca[nca].rot = rot;
					++nca;
				}
			}
		}
	}
	data->nca = nca;
	print_leave();
	return nca;
}

/** @brief 指定された点を中心に任意の点を指定された角度だけ回転させる関数
 *  @param[in] s 回転前の座標
 *  @param[in] o 原点の座標
 *  @param[in] rot 回転角(90度単位)
 *  @param[out] d 回転後の座標
 */
static void rotate(XY *s, XY *o, gint rot, XY *d) {
	gint rot_cos[] = {1,  0, -1,  0};
	gint rot_sin[] = {0,  1,  0, -1};
	print_enter();
	d->x =  rot_cos[rot] * (s->x - o->x) + rot_sin[rot] * (s->y - o->y) + o->x;
	d->y = -rot_sin[rot] * (s->x - o->x) + rot_cos[rot] * (s->y - o->y) + o->y;
	print_leave();
	return;
}

/** @brief 移動可能な格子点を消去する関数
 *  @param[in] data LatFoldDataのポインター
 */
void clear_candidate_atom(LatFoldData *data) {
	print_enter();
	data->nca = 0;
	reset_atom_move(data);
	print_leave();
	return;
}

/** @brief 移動可能な移動先が既知でその場所に移動したとき全体を移動させる関数
 *  @param[in] data LatFoldDataのポインター
 *  @param[in] mouse クリックに関するマウス情報
 *  @param[in] preview プレビューモードかどうかのフラグ
 */
void move_atom(LatFoldData *data, Mouse *mouse, gboolean preview) {
	gint i;
	CandidateAtom *ca;
	Atom *a;
	print_enter();
	data->mode_preview = FALSE;
	for (i = 0; i < data->nca; ++i) {
		ca = &(data->ca[i]);
		if (mouse->a1.x == ca->x && mouse->a1.y == ca->y) {
			try_bend(data, ca->origin, ca->direction, ca->rot);
			if (preview == FALSE) {
				a = data->atom;
				data->atom = data->work;
				data->work = a;
			} else {
				data->mode_preview = TRUE;
			}
			break;
		}
	}
	print_leave();
	return;
}

/** @brief 移動可能な移動先の結合を調べる関数
 *  @param[in] data LatFoldDataのポインター
 *  @param[in] mouse クリックに関するマウス情報
 *  @retval nca 移動可能な結合の数
 */
gint set_candidate_bond_bend(LatFoldData *data, Mouse *mouse) {
	XY *oxy, *ixy, jxy;
	gint i1, i, rot;
	gint nca = 0;
	gint iatom[2], neighbor[2], direction[2];
	i1 = lattice2bond(data->atom, data->natom, &mouse->a1, &mouse->a2);
	if (i1 < 0) {
		data->nca = 0;
		return 0;
	}
	iatom[0] = i1;
	iatom[1] = i1 + 1;
	neighbor[0] = i1 + 1;
	neighbor[1] = i1;
	direction[0] = -1;
	direction[1] = 1;
	for (i = 0; i < 2 ; ++i) {
		ixy = (XY *)&data->atom[iatom[i]];
		if (neighbor[i] >= 0 && neighbor[i] < data->natom) {
			oxy = (XY *)&data->atom[neighbor[i]];
			for (rot = 1; rot < 4; ++rot) {
				rotate(ixy, oxy, rot, &jxy);
				if (try_bend(data, neighbor[i], i, rot)) {
					data->ca[nca].x = jxy.x;
					data->ca[nca].y = jxy.y;
					data->ca[nca].origin = neighbor[i];
					data->ca[nca].direction = direction[i];
					data->ca[nca].rot = rot;
					++nca;
				}
			}
		}
	}
	data->nca = nca;
	print_leave();
	return nca;
}

/** @brief 移動可能な移動先が既知でその場所に移動したとき全体を移動させる関数
 *  @param[in] data LatFoldDataのポインター
 *  @param[in] mouse クリックに関するマウス情報
 *  @param[in] preview プレビューモードかどうかのフラグ
 */
void move_bond_bend(LatFoldData *data, Mouse *mouse, gboolean preview) {
	gint i;
	CandidateAtom *ca;
	XY *oxy;
	Atom *a;
	print_enter();
	data->mode_preview = FALSE;
	for (i = 0; i < data->nca; ++i) {
		ca = &(data->ca[i]);
		oxy = (XY *)&(data->atom[ca->origin]);
		if ((mouse->a1.x == ca->x && mouse->a1.y == ca->y
					&& mouse->a2.x == oxy->x && mouse->a2.y == oxy->y)
				|| (mouse->a2.x == ca->x && mouse->a2.y == ca->y
					&& mouse->a1.x == oxy->x && mouse->a1.y == oxy->y)) {
			try_bend(data, ca->origin, ca->direction, ca->rot);
			if (preview == FALSE) {
				a = data->atom;
				data->atom = data->work;
				data->work = a;
			} else {
				data->mode_preview = TRUE;
			}
			break;
		}
	}
	print_leave();
	return;
}

/** @brief すべてのmoveフラグをリセットする関数
 *  @param[in] data Atom型の配列
 */
static void reset_atom_move(LatFoldData *data) {
	gint i;
	print_enter();
	for (i = 0; i < data->natom; ++i) {
		data->move[i] = FALSE;
	}
	print_leave();
	return;
}

/** @brief 1原子flipによる移動可能な移動先が既知で
 *         その場所に移動したとき全体を移動させる関数
 *  @param[in] data LatFoldDataのポインター
 *  @param[in] mouse クリックに関するマウス情報
 *  @param[in] preview プレビューモードかどうかのフラグ
 */
void move_atom_flip1(LatFoldData *data, Mouse *mouse, gboolean preview) {
	gint i;
	CandidateAtom *ca;
	Atom *a;
	print_enter();
	data->mode_preview = FALSE;
	for (i = 0; i < data->nca; ++i) {
		ca = &(data->ca[i]);
		if (mouse->a1.x == ca->x && mouse->a1.y == ca->y) {
			if (preview == FALSE) {
				a = data->atom;
				data->atom = data->work;
				data->work = a;
			} else {
				data->mode_preview = TRUE;
			}
			break;
		}
	}
	print_leave();
	return;
}

/** @brief 1原子flipによる移動可能な移動先の格子点を調べる関数
 *  @param[in] data LatFoldDataのポインター
 *  @param[in] mouse クリックに関するマウス情報
 *  @retval nca 移動可能な格子点の数
 */
gint set_candidate_flip1(LatFoldData *data, Mouse *mouse) {
	gint iatom;
	Atom *n1, *n2, *ia;
	XY l;
	print_enter();
	data->nca = 0;
	iatom = lattice2atom(data->atom, data->natom, &mouse->a1);
	if (iatom < 0 || iatom == 0 || iatom == data->natom - 1) {
		return 0;
	}
	copy_atom(data->atom, data->work, data->natom);
	n1 = &(data->atom[iatom - 1]);
	n2 = &(data->atom[iatom + 1]);
	ia = &(data->atom[iatom]);
	if ((ia->x == n1->x && ia->x == n2->x) || (ia->y == n1->y && ia->y == n2->y)) {
		/* 直線上に並んだらflipできないので抜ける */
		return 0;
	}
	if (ia->x == n1->x && ia->y == n2->y) {
		data->ca[0].x = n2->x;
		data->ca[0].y = n1->y;
	} else if (ia->y == n1->y && ia->x == n2->x) {
		data->ca[0].x = n1->x;
		data->ca[0].y = n2->y;
	}
	l.x =  data->ca[0].x;
	l.y =  data->ca[0].y;
	if (lattice2atom(data->atom, data->natom, &l) >= 0) {
		/* 移動先に原子が存在したら抜ける */
		return 0;
	}
	data->ca[0].direction = 0;
	data->work[iatom].x = data->ca[0].x;
	data->work[iatom].y = data->ca[0].y;
	data->move[iatom] = TRUE;
	data->nca = 1;
	print_leave();
	return 1;
}

