#include "latfold.h"

static void load_new_game(GtkAction *action, LatFold *lf);
static void revert(GtkAction *action, LatFold *lf);
static void show_answer(GtkAction *action, LatFold *lf);
static void fullscreen(GtkToggleAction *action, LatFold *lf);

/** @brief newメニューが呼ばれたときに呼ばれる関数
 *  @param[in] action アクション
 *  @param[in] lf LatFoldのデータ
 */
static void load_new_game(GtkAction *action, LatFold *lf) {
	LatFoldGUI *gui = lf->gui;
	print_enter();
	g_return_if_fail(action != NULL);
	/* ステータスバーの中身を空にする */
	clear_statusbar(gui);
	/* 新規ダイアローグを開く */
	open_new_dialog(lf);
	/* 描画の準備と描画 */
	create_da_work(gui);
	create_mask(gui);
	create_tile(gui);
	create_background(gui);
	draw(lf);
	print_leave();
	return;
}

/** @brief revertメニューが呼ばれたときに呼ばれる関数
 *  @param[in] action アクション
 *  @param[in] lf LatFoldのデータ
 */
static void revert(GtkAction *action, LatFold *lf) {
	print_enter();
	g_return_if_fail(action != NULL);
	/* ステータスバーのメッセージを消去する */
	clear_statusbar(lf->gui);
	/* 分子を初期状態に戻す */
	if (lf->data != NULL) {
		make_molecule(lf->data, lf->data->seed);
	}
	/* センタリングを行う */
	lf->gui->wc.x = 0;
	lf->gui->wc.y = 0;
	create_background(lf->gui);
	draw(lf);
	print_leave();
	return;
}

/** @brief solutionメニューが呼ばれたときに呼ばれる関数
 *  @param[in] action アクション
 *  @param[in] lf LatFoldのデータ
 */
static void show_answer(GtkAction *action, LatFold *lf) {
	LatFoldGUI *gui;
	gint i;
	g_return_if_fail(action != NULL);
	gui = lf->gui;
	/* ステータスバーにメッセージを表示させる */
	if (lf->gui->tag_monitor > 0) {
		/* 探索途中の場合 */
		set_statusbar(lf->gui, _("The best solution which is found until now is shown."));
	} else {
		/* 探索が終了している場合 */
		set_statusbar(lf->gui, _("One of the best solutions is shown."));
	}
	/* センタリングを行う */
	gui->wc.x = 0;
	gui->wc.y = 0;
	create_background(gui);
	/* 最善解をコピーする */
	if (lf->data != NULL && lf->data->solve != NULL) {
		for (i = 0; i < lf->data->natom; ++i) {
			lf->data->atom[i] = lf->data->solve->best[i];
		}
	}
	draw(lf);
	lf->gui->operatable = FALSE;
	return;
}

/** @brief fullscreenメニューが呼ばれたときに呼ばれる関数
 *  @param[in] action アクション
 *  @param[in] lf LatFoldのデータ
 */
static void fullscreen(GtkToggleAction *action, LatFold *lf) {
	if (gtk_toggle_action_get_active(action)) {
		gtk_window_fullscreen(GTK_WINDOW(lf->gui->main));
	} else {
		gtk_window_unfullscreen(GTK_WINDOW(lf->gui->main));
	}
	return;
}

/** @brief メニューを作る関数
 *  @param[in] lf LatFoldのポインター
 *  @return メニューのウィジェット
 */
GtkWidget *create_menu(LatFold *lf) {
	print_enter();
	GtkUIManager *ui;
	GtkActionGroup *actions;
	GError *error = NULL;
	const gchar *ui_info = 
		"<ui>"
		"  <menubar name='MenuBar'>"
		"    <menu action='GameMenu'>"
		"      <menuitem action='New'/>"
		"      <menuitem action='Revert'/>"
		"      <separator/>"
		"      <menuitem action='Solution'/>"
		"      <separator/>"
		"      <menuitem action='Quit'/>"
		"    </menu>"
		"    <menu action='ViewMenu'>"
		"      <menuitem action='Fullscreen'/>"
		"    </menu>"
		"    <menu action='HelpMenu'>"
		"      <menuitem action='Help'/>"
		"      <separator/>"
		"      <menuitem action='About'/>"
		"    </menu>"
		"  </menubar>"
		"</ui>";
	GtkActionEntry entries[] = {
		{ "GameMenu", NULL, _("Game") },	/* name, stock id, label */
		{ "New", GTK_STOCK_NEW,			/* name, stock id */
			_("New"), "<control>N",		/* label, accelerator */
			_("Create a new game"),		/* tooltip */
			G_CALLBACK(load_new_game) },
		{ "Revert", GTK_STOCK_REVERT_TO_SAVED,	/* name, stock id */
			_("Revert"), "<control>R",	/* label, accelerator */
			_("Revert game"),				/* tooltip */
			G_CALLBACK(revert) },
		{ "Solution", GTK_STOCK_HELP,	/* name, stock id */
			_("Solution"), "<control>A",	/* label, accelerator */
			_("Show best form"),				/* tooltip */
			G_CALLBACK(show_answer) },
		{ "Quit", GTK_STOCK_QUIT,		/* name, stock id */
			_("Quit"), "<control>Q",	/* label, accelerator */
			_("Quit game"),				/* tooltip */
			G_CALLBACK(gtk_main_quit) },
		{ "ViewMenu", NULL, _("View") },	/* name, stock id, label */
		{ "HelpMenu", NULL, _("Help") },	/* name, stock id, label */
		{ "Help", GTK_STOCK_HELP,			/* name, stock id */
			_("Help Contents"), NULL,		/* label, accelerator */
			_("About instruction and score"),		/* tooltip */
			G_CALLBACK(help_dialog) },
		{ "About", GTK_STOCK_ABOUT,			/* name, stock id */
			_("About"), NULL,		/* label, accelerator */
			_("About this game"),		/* tooltip */
			G_CALLBACK(open_about_dialog) },
	};
	GtkToggleActionEntry toggle_entries[] = {
		{ "Fullscreen", GTK_STOCK_FULLSCREEN,		/* name, stock id */
			_("Full screen"), "F11",	/* label, accelerator */
			_("Full Screen Mode"),				/* tooltip */
			G_CALLBACK(fullscreen) },
	};
	guint n_entries = G_N_ELEMENTS(entries);
	guint n_toggle_entries = G_N_ELEMENTS(toggle_entries);
	GtkAccelGroup *ui_manager;
	actions = gtk_action_group_new("Actions");
	gtk_action_group_add_actions(actions, entries, n_entries, lf);
	gtk_action_group_add_toggle_actions(actions, toggle_entries, n_toggle_entries, lf);
	ui = gtk_ui_manager_new();
	gtk_ui_manager_insert_action_group(ui, actions, 0);
	g_object_unref(actions);
	ui_manager = gtk_ui_manager_get_accel_group(ui);
	gtk_window_add_accel_group(GTK_WINDOW(lf->gui->main), ui_manager);
	if (!gtk_ui_manager_add_ui_from_string(ui, ui_info, -1, &error)) {
		g_message("building menus failed: %s", error->message);
		g_error_free (error);
	}
	lf->gui->action_fullscreen = gtk_action_group_get_action(actions, "Fullscreen");
	print_leave();
	return gtk_ui_manager_get_widget(ui, "/MenuBar");
}
