#include "latfold.h"

static void expose(GtkWidget *widget, GdkEventExpose *event, LatFold *lf);
static void resize(GtkWidget *widget, GdkEventConfigure *event, LatFold *lf);
static void detect_motion_event(LatFold *lf);
static void get_mouse(LatFold *lf, GdkEventButton *event, Mouse *mouse);
static void reset_mouse(Mouse *mouse);
static void reset_selection(LatFold *lf);
static OperationType get_operation(gint button, gint position);
static gboolean same_position(Mouse *m1, Mouse *m2);
static void da_press(GtkWidget *widget, GdkEventButton *event, LatFold *lf);
static void da_release(GtkWidget *widget, GdkEventButton *event, LatFold *lf);
static void da_motion(GtkWidget *widget, GdkEventButton *event, LatFold *lf);
static void draw_molecule(LatFold *lf, Atom *atom, gboolean preview);
static void draw_bond_in_lattice(LatFoldGUI *gui, GdkPixmap *pixmap,
		GdkGC *gc_solid, GdkGC *gc_line, XY *l1, XY *l2);
static void draw_bond_in_widget(LatFoldGUI *gui, GdkPixmap *pixmap,
		GdkGC *gc_solid, GdkGC *gc_line, XY *w1, XY *w2);
static void draw_atom_in_lattice(LatFoldGUI *gui, GdkPixmap *pixmap,
		GdkGC *gc_solid, GdkGC *gc_line, XY *l);
static void draw_atom_in_widget(LatFoldGUI *gui, GdkPixmap *pixmap,
		GdkGC *gc_solid, GdkGC *gc_line, XY *c);
static void create_gc_tile(LatFoldGUI *gui);
static GdkGC *create_color(LatFoldGUI *gui, gdouble r, gdouble g, gdouble b);

static void (*func_move[])(LatFoldData *, Mouse *, gboolean) = {
	NULL, /* NONE */
	NULL, /* TRANSLATION */
	move_atom, /* BEND */
	move_bond_bend, /* BEND_BOND */
	move_atom_flip1 /* FLIP1 */
};

static gint (*func_candidate[])(LatFoldData *, Mouse *mouse) = {
	NULL, /* NONE */        
	NULL, /* TRANSLATION */ 
	set_candidate_atom, /* BEND */
	set_candidate_bond_bend, /* BEND_BOND */
	set_candidate_flip1 /* FLIP1 */
};

/** @brief Drawing Areaの作成と初期設定を行う関数
 *  @param[in] lf LatFoldのポインター
 */
GtkWidget *latfold_gui_da_new(LatFold *lf) {
	GtkWidget *w;
	LatFoldGUI *gui;
	print_enter();
	gui = lf->gui;
	gui->da = w = gtk_drawing_area_new();
	gtk_widget_set_size_request(w, 300, 300);
	gtk_widget_add_events(w, 
			GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK
			| GDK_POINTER_MOTION_MASK);
	g_signal_connect(G_OBJECT(w), "expose_event",
			G_CALLBACK(expose), lf);
	g_signal_connect(G_OBJECT(w), "configure_event",
			G_CALLBACK(resize), lf);
	g_signal_connect(G_OBJECT(w), "button_press_event",
			G_CALLBACK(da_press), lf);
	g_signal_connect(G_OBJECT(w), "button_release_event",
			G_CALLBACK(da_release), lf);
	g_signal_connect(G_OBJECT(w), "motion_notify_event",
			G_CALLBACK(da_motion), lf);
	print_leave();
	return w;
}

/** @brief expose eventを処理する。
 *  @param[in] widget Drawing Areaのwidget
 *  @param[in] event Drawing Areaで発生したevent
 *  @param[in] lf LatFoldのポインター
 */
static void expose(GtkWidget *widget, GdkEventExpose *event, LatFold *lf) {
	print_enter();
	g_return_if_fail(widget != NULL);
	g_return_if_fail(event != NULL);
	draw(lf);
	print_leave();
	return;
}

/** @brief configure event(resize)を処理する。
 *  @param[in] widget Drawing Areaのwidget
 *  @param[in] event Drawing Areaで発生したevent
 *  @param[in] lf LatFoldのポインター
 */
static void resize(GtkWidget *widget, GdkEventConfigure *event, LatFold *lf) {
	static gboolean first = TRUE;
	print_enter();
	g_return_if_fail(widget != NULL);
	g_return_if_fail(event != NULL);
	if (first) {
		first = FALSE;
		print_leave();
		return;
	}
	create_da_work(lf->gui);
	create_mask(lf->gui);
	create_tile(lf->gui);
	create_background(lf->gui);
	print_leave();
	return;
}

/** @brief motionイベントを検出できるようにする問題回避用の関数
 *  @param[in] lf LatFoldのポインター
 */
static void detect_motion_event(LatFold *lf) {
	LatFoldGUI *g = lf->gui;
	if (g->tag_idle > 0) {
		g_source_remove(g->tag_idle);
	}
	g->tag_idle = g_idle_add((GSourceFunc)idle, lf);
	return;
}

/** @brief マウスによる原子または結合のクリックを解除する関数
 *  @param[out] mouse Selectionのポインター
 */
static void reset_mouse(Mouse *mouse) {
	mouse->mtype = MOUSE_TYPE_NONE;
	mouse->otype = NONE;
	return;
}

/** @brief マウスによる原子の選択をリセットする関数
 *  @param[in] lf LatFoldのポインター
 */
static void reset_selection(LatFold *lf) {
	lf->gui->selected = FALSE;
	lf->data->mode_preview = FALSE;
	clear_candidate_atom(lf->data);
	lf->data->op_type = NONE;
	draw(lf);
	return;
}

/** @brief マウスのボタン番号とクリック箇所から操作種類に変換する関数
 *  @param[in] button マウスのボタン番号
 *  @param[in] position クリックされた箇所（9方向）
 *  @retval op_type 操作種類
 */
static OperationType get_operation(gint button, gint position) {
	OperationType op_type = NONE;
	switch (button) {
		case 1:
			if (position == 5) {
				op_type = FLIP1;
			}
			if (position % 2 == 0) {
				op_type = BEND_BOND;
			}
			break;
		case 2:
			if (position == 5) {
				op_type = BEND;
			}
			break;
		case 3:
			op_type = TRANSLATION;
		default:
			;
	}
	return op_type;
}

/** @brief マウスに関する情報を収集する関数
 *  @param[in] lf LatFoldのポインター
 *  @param[in] event GdkEventButtonのポインター
 *  @param[out] mouse Selectionのポインター
 */
static void get_mouse(LatFold *lf, GdkEventButton *event, Mouse *mouse) {
	XY e, lxy;
	gint position;
	e.x = (gint)event->x;
	e.y = (gint)event->y;
	lattice_position(lf, e, &lxy, &position);
	mouse->otype = get_operation(event->button, position);
	mouse->a1.x = lxy.x;
	mouse->a1.y = lxy.y;
	mouse->position = position;
	if (position == 5) {
		mouse->mtype = MOUSE_TYPE_ATOM;
	} else if (position % 2 == 0) {
		mouse->mtype = MOUSE_TYPE_BOND;
		switch (position) {
			case 2:
				mouse->a2.x = lxy.x;
				mouse->a2.y = lxy.y+1;
				break;
			case 4:
				mouse->a2.x = lxy.x-1;
				mouse->a2.y = lxy.y;
				break;
			case 6:
				mouse->a2.x = lxy.x+1;
				mouse->a2.y = lxy.y;
				break;
			case 8:
				mouse->a2.x = lxy.x;
				mouse->a2.y = lxy.y-1;
				break;
			default:
				g_warning("Unknown position %d\n", position);
				break;
		}
	} else {
		mouse->mtype = MOUSE_TYPE_NONE;
	}
	return;
}

/** @brief ボタンが押されたのとほぼ同一場所かどうかの判定をする関数
 *  @param[in] m1 Mouseのポインター
 *  @param[in] m2 Mouseのポインター
 *  @retval TRUE 同一の時
 *  @retval FALSE 同一でない時
 */
static gboolean same_position(Mouse *m1, Mouse *m2) {
	if (m1->mtype != m2->mtype) {
		return FALSE;
	}
	if (m1->otype != m2->otype) {
		return FALSE;
	}
	switch (m1->mtype) {
		case MOUSE_TYPE_NONE:
			return FALSE;
		case MOUSE_TYPE_ATOM:
			if (m1->a1.x == m2->a1.x && m1->a1.y == m2->a1.y) {
				return TRUE;
			} else {
				return FALSE;
			}
		case MOUSE_TYPE_BOND:
			if (m1->a1.x == m2->a1.x && m1->a1.y == m2->a1.y
					&& m1->a2.x == m2->a2.x && m1->a2.y == m2->a2.y) {
				return TRUE;
			} else if (m1->a1.x == m2->a2.x && m1->a1.y == m2->a2.y
					&& m1->a2.x == m2->a1.x && m1->a2.y == m2->a1.y) {
				return TRUE;
			} else {
				return FALSE;
			}
	}
	g_error("Error occured in same_position()\n");
}

/** @brief マウスのボタンが押されたときに呼ばれる関数
 *  @param[in] widget Drawing Areaのwidget
 *  @param[in] event Drawing Areaで発生したevent
 *  @param[in] lf LatFoldのポインター
 */
static void da_press(GtkWidget *widget, GdkEventButton *event, LatFold *lf) {
	LatFoldGUI *g;
	Mouse mouse;
	print_enter();
	g_return_if_fail(widget != NULL);
	g = lf->gui;
	/* idleがセットされているとmotionをなぜか検知できないのでその対策 */
	if (g->tag_idle > 0) {
		g_source_remove(g->tag_idle);
		g->tag_idle = 0;
	}
	/* 描くものが無ければ、何もしない */
	if (lf->data == NULL) {
		print_leave();
		return;
	}
	get_mouse(lf, event, &mouse);
	if (mouse.otype == TRANSLATION) {
		g->last.x = (gint)event->x;
		g->last.y = (gint)event->y;
		g->translation = TRUE;
	} else {
		if (g->operatable) {
			g->press = mouse;
		}
	}
	print_leave();
	return;
}

/** @brief マウスのボタンが離されたときに呼ばれる関数
 *  @param[in] widget Drawing Areaのwidget
 *  @param[in] event Drawing Areaで発生したevent
 *  @param[in] lf LatFoldのポインター
 */
static void da_release(GtkWidget *widget, GdkEventButton *event, LatFold *lf) {
	LatFoldGUI *g;
	LatFoldData *d;
	Mouse mouse;
	print_enter();
	g_return_if_fail(widget != NULL);
	g = lf->gui;
	d = lf->data;
	/* idle関数の再設定 */
	detect_motion_event(lf);
	/* 描くものが無ければ、何もしない */
	if (d == NULL) {
		print_leave();
		return;
	}
	get_mouse(lf, event, &mouse);
	if (mouse.otype == TRANSLATION) {
		g->wc.x -= (gint)event->x - g->last.x;
		g->wc.y -= (gint)event->y - g->last.y;
		create_background(g);
		draw(lf);
		g->translation = FALSE;
	} else if (mouse.otype == NONE) {
		reset_selection(lf);
	} else if (same_position(&g->press, &mouse)) {
		/* ボタンが押されたのとほぼ同じ場所の場合 */
		if (!g->selected) {
			/* 未選択状態の場合 */
			g->click1 = mouse;
			if (g->click1.mtype == MOUSE_TYPE_ATOM) {
				/* 原子を選択した場合 */
				if (lattice2atom(d->atom, d->natom, &mouse.a1) >= 0) {
					d->op_type = g->click1.otype;
					if (func_candidate[g->click1.otype](d, &mouse) > 0) {
						g->selected = TRUE;
						g->selected_l = mouse.a1;
					} else {
						g->selected = FALSE;
					}
					draw(lf);
				} else {
					reset_mouse(&g->click1);
				}
			} else if (g->click1.mtype == MOUSE_TYPE_BOND) {
				gint ia = lattice2bond(d->atom, d->natom, &mouse.a1, &mouse.a2);
				if (ia >= 0) {
					d->op_type = g->click1.otype;
					if (func_candidate[g->click1.otype](d, &mouse) > 0) {
						g->selected = TRUE;
						g->selected_l.x = d->atom[ia].x;
						g->selected_l.y = d->atom[ia].y;
					} else {
						g->selected = FALSE;
					}
					draw(lf);
				} else {
					reset_mouse(&g->click1);
				}
			} else {
				reset_mouse(&g->click1);
			}
		} else {
			/* 選択状態の場合 */
			g->click2 = mouse;
			if (candidate_place(d->ca, d->nca, &g->click2) >= 0) {
				/* 移動可能な場所を選択した場合 */
				gint current_score;
				clear_statusbar(g);
				func_move[d->op_type](d, &g->click2, FALSE);
				current_score = calc_score(d, d->atom);
				set_score(g, current_score);
				if (current_score > d->user_best) {
					gdouble t;
					gint i;
					t = g_timer_elapsed(d->user_timer, NULL);
					for (i = d->user_best; i < current_score; ++i) {
						gchar *str;
						str = g_strdup_printf("%.4f", t);
						latfold_gui_clist_set_score(g, i+1, str, TRUE);
						g_nullify_pointer((gpointer)&str);
					}
					if (current_score >= d->solve->max_score) {
						set_statusbar(g, _("Complete!"));
					}
					d->user_best = current_score;
				}
			}
			reset_mouse(&g->click1);
			reset_mouse(&g->click2);
			reset_selection(lf);
		}
	} else {
			reset_mouse(&g->click1);
			reset_mouse(&g->click2);
			reset_selection(lf);
	}
	reset_mouse(&g->press);
	print_leave();
	return;
}

/** @brief マウスを動かしたときに呼ばれる関数
 *  @param[in] widget Drawing Areaのwidget
 *  @param[in] event Drawing Areaで発生したevent
 *  @param[in] lf LatFoldのポインター
 */
static void da_motion(GtkWidget *widget, GdkEventButton *event, LatFold *lf) {
	LatFoldGUI *g;
	Mouse mouse;
	XY e;
	print_enter();
	g_return_if_fail(widget != NULL);
	/* 描くものが無ければ、何もしない */
	if (lf->data == NULL) {
		print_leave();
		return;
	}
	g = lf->gui;
	get_mouse(lf, event, &mouse);
	e.x = (gint)event->x;
	e.y = (gint)event->y;
	/* イベント待ちがある場合は、motionイベントを無視する */
	if (gtk_events_pending()) {
		print_leave();
		return;
	}
	if (g->translation) {
		g->wc.x -= e.x - g->last.x;
		g->wc.y -= e.y - g->last.y;
		g->last.x = e.x;
		g->last.y = e.y;
		create_background(g);
		draw(lf);
	} else if (g->click1.otype == BEND || g->click1.otype == FLIP1) {
		if (mouse.position == 5) {
			func_move[g->click1.otype](lf->data, &mouse, TRUE);
		} else {
			lf->data->mode_preview = FALSE;
		}
		draw(lf);
	} else if (g->click1.otype == BEND_BOND) {
		if (mouse.position % 2 == 0) {
			func_move[g->click1.otype](lf->data, &mouse, TRUE);
		} else {
			lf->data->mode_preview = FALSE;
		}
		draw(lf);
	}
	print_leave();
	return;
}

/** @brief PixmapのデータをDrawing Areaに描画する。
 *  @param[in] lf LatFoldのポインター
 */
void draw(LatFold *lf) {
	LatFoldGUI *gui;
	LatFoldData *data;
	GtkWidget *widget;
	gint i;
	print_enter();
	gui = lf->gui;
	data = lf->data;
	widget = gui->da;
	/* 背景pixmapを完成版のpixmapへコピーする */
	gdk_draw_drawable(gui->product,
			gui->da->style->fg_gc[GTK_WIDGET_STATE(gui->da)],
			gui->background,
			0, 0, 0, 0, gui->pixcel_max.x, gui->pixcel_max.y);
	/* データが無ければ描くものもない */
	if (data != NULL) {
		/* 分子を描く */
		draw_molecule(lf, data->atom, FALSE);
		switch (gui->click1.mtype) {
			case MOUSE_TYPE_ATOM:
				/* 原子の移動先候補があれば選択原子を描く */
				if (data->nca > 0 && gui->selected) {
					draw_atom_in_lattice(gui, gui->product, NULL,
							gui->gc_yellow, &gui->selected_l);
				}
				/* 原子の移動先候補があれば候補を描く */
				for (i = 0; i < data->nca; ++i) {
					draw_atom_in_lattice(gui, gui->product, gui->gc_black,
							gui->gc_yellow, (XY *)&data->ca[i]);
				}
				break;
			case MOUSE_TYPE_BOND:
				/* 結合の移動先候補があれば、選択した結合を描く */
				if (data->nca > 0 && gui->selected) {
					draw_bond_in_lattice(gui, gui->product, NULL,
							gui->gc_yellow, &gui->click1.a1, &gui->click1.a2);
					draw_bond_in_lattice(gui, gui->product, NULL,
							gui->gc_yellow, &gui->click1.a2, &gui->click1.a1);
				}
				/* 結合の移動先候補があれば候補を描く */
				for (i = 0; i < data->nca; ++i) {
					XY *l1, *l2;
					l1 = (XY *)&data->ca[i];
					l2 = (XY *)&data->atom[data->ca[i].origin];
					draw_bond_in_lattice(gui, gui->product, gui->gc_black,
							gui->gc_yellow, l1, l2);
					draw_bond_in_lattice(gui, gui->product, gui->gc_black,
							gui->gc_yellow, l2, l1);
				}
				break;
		}
		/* プレビューを描く */
		if (data->mode_preview == TRUE) {
			draw_molecule(lf, data->work, TRUE);
		}
	}
	/* 完成したPixmapをDrawingAreaに描く */
	gdk_draw_drawable(widget->window,
			widget->style->fg_gc[GTK_WIDGET_STATE(widget)],
			gui->product,
			0, 0, 0, 0, gui->pixcel_max.x, gui->pixcel_max.y);
	print_leave();
	return;
}

/** @brief 分子を描く関数
 *  @param[in] lf LatFoldのポインター
 *  @param[in] atom Atom配列
 *  @param[in] preview プレビューかどうか
 */
static void draw_molecule(LatFold *lf, Atom *atom, gboolean preview) {
	LatFoldGUI *gui;
	LatFoldData *data;
	print_enter();
	gui = lf->gui;
	data = lf->data;
	gint i;
	/* 結合をPixmapに描く */
	for (i = 0; i < data->natom; ++i) {
		GdkGC *color, *outline;
		XY *l1, *l2;
		gint neighbor;
		if (atom[i].type == ATOM_P) {
			color = gui->gc_blue;
		} else {
			color = gui->gc_red;
		}
		l1 = (XY *)&atom[i];
		/* 中心点から半分ずつ結合を描く */
		for (neighbor = i-1; neighbor < i+2; neighbor += 2) {
			if (neighbor >= 0 && neighbor < data->natom) {
				l2 = (XY *)&atom[neighbor];
				if (data->mode_preview == TRUE
						&& (data->move[i] == TRUE
							|| data->move[neighbor] == TRUE)) {
					if (preview == TRUE) {
						outline = gui->gc_yellow;
					} else {
						outline = gui->gc_gray;
					}
				} else {
					outline = gui->gc_white;
				}
				draw_bond_in_lattice(gui, gui->product, color, outline, l1, l2);
			}
		}
	}
	/* 原子をPixmapに描く */
	for (i = 0; i < data->natom; ++i) {
		GdkGC *color, *outline;
		if (atom[i].type == ATOM_P) {
			color = gui->gc_blue;
		} else {
			color = gui->gc_red;
		}
		if (data->mode_preview == TRUE && data->move[i] == TRUE) {
			if (preview == TRUE) {
				outline = gui->gc_yellow;
			} else {
				outline = gui->gc_gray;
			}
		} else {
			outline = gui->gc_white;
		}
		draw_atom_in_lattice(gui, gui->product, color, outline, (XY *)&atom[i]);
	}
}

/** @brief Lattice座標系で格子間結合を半分描く関数
 *  @param[in] gui LatFoldGUIのポインター
 *  @param[in] pixmap 描く対象のPixmap
 *  @param[in] gc_solid 塗りつぶしに用いるGC
 *  @param[in] gc_line アウトラインに用いるGC
 *  @param[in] l1 結合の原点となるLattice座標
 *  @param[in] l2 結合の方向となるLattice座標
 */
static void draw_bond_in_lattice(LatFoldGUI *gui, GdkPixmap *pixmap,
		GdkGC *gc_solid, GdkGC *gc_line, XY *l1, XY *l2) {
	XY w1, w2;
	lattice2widget(gui, l1, &w1);
	lattice2widget(gui, l2, &w2);
	draw_bond_in_widget(gui, pixmap, gc_solid, gc_line, &w1, &w2);
	print_leave();
	return;
}

/** @brief Widget座標系で格子間結合を半分描く関数
 *  @param[in] gui LatFoldGUIのポインター
 *  @param[in] pixmap 描く対象のPixmap
 *  @param[in] gc_solid 塗りつぶしに用いるGC
 *  @param[in] gc_line アウトラインに用いるGC
 *  @param[in] w1 結合の原点となるWidget座標
 *  @param[in] w2 結合の方向となるWidget座標
 */
static void draw_bond_in_widget(LatFoldGUI *gui, GdkPixmap *pixmap,
		GdkGC *gc_solid, GdkGC *gc_line, XY *w1, XY *w2) {
	gint x, y;
	gint j;
	BondDirection m;
	GdkGC *gc[2];
	print_enter();
	gc[BOND_SOLID] = gc_solid;
	gc[BOND_LINE] = gc_line;
	if (w1->x == w2->x) {
		if (w1->y < w2->y) {
			m = BOND_BOTTOM;
		} else {
			m = BOND_TOP;
		}
	} else if (w1->y == w2->y) {
		if (w1->x < w2->x) {
			m = BOND_RIGHT;
		} else {
			m = BOND_LEFT;
		}
	} else {
		/* ここには来ないはず */
		return;
	}
	x = w1->x - gui->tilesize/2;
	y = w1->y - gui->tilesize/2;
	for (j = 0; j < 2; ++j) {
		/* 塗りつぶし、輪郭の順に結合を描く */
		if (gc[j] == NULL) {
			continue;
		}
		gdk_gc_set_clip_mask(gc[j], gui->mask[m][j]);
		gdk_gc_set_clip_origin(gc[j], x, y);
		gdk_draw_rectangle(pixmap, gc[j], TRUE, x, y,
				gui->tilesize, gui->tilesize);
		gdk_gc_set_clip_origin(gc[j], 0, 0);
		gdk_gc_set_clip_mask(gc[j], NULL);
	}
	print_leave();
	return;
}

/** @brief Lattice座標系で原子を描く関数
 *  @param[in] gui LatFoldGUIのポインター
 *  @param[in] pixmap 描く対象のPixmap
 *  @param[in] gc_solid 塗りつぶしに用いるGCでNULLなら塗りつぶさない
 *  @param[in] gc_line アウトラインに用いるGCでNULLなら輪郭を省く
 *  @param[in] l 中心のLattice座標系の座標
 */
static void draw_atom_in_lattice(LatFoldGUI *gui, GdkPixmap *pixmap,
		GdkGC *gc_solid, GdkGC *gc_line, XY *l) {
	XY w;
	print_enter();
	lattice2widget(gui, l, &w);
	draw_atom_in_widget(gui, pixmap, gc_solid, gc_line, &w);
	print_leave();
	return;
}

/** @brief Widget座標系で原子を描く関数
 *  @param[in] gui LatFoldGUIのポインター
 *  @param[in] pixmap 描く対象のPixmap
 *  @param[in] gc_solid 塗りつぶしに用いるGCでNULLなら塗りつぶさない
 *  @param[in] gc_line アウトラインに用いるGCでNULLなら輪郭を省く
 *  @param[in] c 中心のWidget座標
 */
static void draw_atom_in_widget(LatFoldGUI *gui, GdkPixmap *pixmap,
		GdkGC *gc_solid, GdkGC *gc_line, XY *c) {
	gint ar, ad;
	print_enter();
	ar = iround(gui->atomradius*gui->zoom);
	ad = ar + ar;
	if (gc_solid != NULL) {
		gdk_draw_arc(pixmap, gc_solid, TRUE,
				c->x-ar, c->y-ar, ad, ad, 64*0, 64*360);
	}
	if (gc_line != NULL) {
		gdk_draw_arc(pixmap, gc_line, FALSE,
				c->x-ar, c->y-ar, ad, ad, 64*0, 64*360);
	}
	print_leave();
	return;
}

/** @brief DrawingAreaのサイズが変わったときにwork用のpixmapを作りなおす関数
 *  @param[in] gui LatFoldGUIのポインター
 */
void create_da_work(LatFoldGUI *gui) {
	print_enter();
	gui->pixcel_max.x = gui->da->allocation.width;
	gui->pixcel_max.y = gui->da->allocation.height;
	if (gui->background != NULL) {
		g_object_unref(gui->background);
		gui->background = NULL;
	}
	if (gui->product != NULL) {
		g_object_unref(gui->product);
		gui->product = NULL;
	}
	gui->background = gdk_pixmap_new(gui->da->window,
			gui->pixcel_max.x, gui->pixcel_max.y, -1);
	gui->product = gdk_pixmap_new(gui->da->window,
			gui->pixcel_max.x, gui->pixcel_max.y, -1);
	print_leave();
	return;
}

/** @brief backgroundのタイル用のGCを設定する。
 *  @param[in] gui LatFoldGUIのポインター
 */
void create_tile(LatFoldGUI *gui) {
	GdkPixmap *tile;
	XY wo, wd[4];
	gint tile_size, tile_center;
	gint i;
	print_enter();
	/* 格子1つ分のピクセル単位のサイズを得る */
	tile_size = gui->tilesize;
	tile_center = tile_size / 2;
	/* タイル用のpixmapを作る */
	tile = gdk_pixmap_new(gui->da->window, tile_size, tile_size, -1);
	/* 背景のpixmapを描く */
	gdk_draw_rectangle(tile,
			gui->da->style->fg_gc[GTK_WIDGET_STATE(gui->da)],
			TRUE, 0, 0, tile_size, tile_size);
	/* 結合用の背景を描く */
	wo.x = tile_center;
	wo.y = tile_center;
	wd[0].x = tile_center;
	wd[0].y = 0;
	wd[1].x = tile_center;
	wd[1].y = tile_size;
	wd[2].x = 0;
	wd[2].y = tile_center;
	wd[3].x = tile_size;
	wd[3].y = tile_center;
	for (i = 0; i < 4; ++i) {
		draw_bond_in_widget(gui, tile, gui->gc_black, gui->gc_gray, &wo, wd+i);
	}
	/* tileに格子点の原子を描く */
	draw_atom_in_widget(gui, tile, gui->gc_black, gui->gc_gray, &wo);
	/* tileをGCに設定する */
	gdk_gc_set_tile(gui->gc_tile, tile);
	/* タイル用のpixmapを壊す */
	if (gui->tile != NULL) {
		g_object_unref(gui->tile);
		gui->tile = NULL;
	}
	gui->tile = tile;
	print_leave();
	return;
}

/** @brief 最初のExpose時にPixmapバッファーを初期化する。
 *  @param[in] gui LatFoldGUIのポインター
 */
void create_background(LatFoldGUI *gui) {
	XY l, w;
	gint offset;
	print_enter();
	l.x = 0;
	l.y = 0;
	offset = iround(gui->tilesize / 2.0);
	lattice2widget(gui, &l, &w);
	gdk_gc_set_ts_origin(gui->gc_tile, offset + w.x, offset + w.y);
	/* 背景のpixmapを描く */
	gdk_draw_rectangle(gui->background, gui->gc_tile, TRUE,
			0, 0, gui->pixcel_max.x, gui->pixcel_max.y);
	print_leave();
	return;
}

/** @brief タイル用GCを作成し、全面色を灰色にする関数
 *  @param gui LatFoldGUIのポインター
 */
static void create_gc_tile(LatFoldGUI *gui) {
	print_enter();
	gui->gc_tile = create_color(gui, 0.5, 0.5, 0.5);
	gdk_gc_set_fill(gui->gc_tile, GDK_TILED);
	print_leave();
	return;
}

/** @brief RBGで指定された色を持つ新しいGCを作成する関数
 *  @param[in] gui LatFoldGUIのポインター
 *  @param[in] r Redの値(0-1)
 *  @param[in] g Greenの値(0-1)
 *  @param[in] b Blueの値(0-1)
 *  @return 指定された色の新しいGC
 */
static GdkGC *create_color(LatFoldGUI *gui, gdouble r, gdouble g, gdouble b) {
	GdkColor color;
	GdkGC *gc;
	print_enter();
	color.red = (guint16)(65535 * r);
	color.green = (guint16)(65535 * g);
	color.blue = (guint16)(65535 * b);
	gc = gdk_gc_new(gui->da->window);
	gdk_gc_set_rgb_fg_color(gc, &color);
	print_leave();
	return gc;
}

/** @brief 色それぞれのGCを作成する関数
 *  @param[in] gui LatFoldGUIのポインター
 */
void create_gc(LatFoldGUI *gui) {
	print_enter();
	create_gc_tile(gui);
	gui->gc_black = create_color(gui, 0.0, 0.0, 0.0);
	gui->gc_gray = create_color(gui, 0.5, 0.5, 0.5);
	gui->gc_blue = create_color(gui, 0.0, 0.0, 1.0);
	gui->gc_red = create_color(gui, 1.0, 0.0, 0.0);
	gui->gc_yellow = create_color(gui, 1.0, 1.0, 0.0);
	gui->gc_white = create_color(gui, 1.0, 1.0, 1.0);
	print_leave();
	return;
}

/** @brief 結合用のビットマップマスクを作成する関数
 *  @param[in] gui LatFoldGUIのポインター
 */
void create_mask(LatFoldGUI *gui) {
	gint i, j;
	gint ar, ad;
	gint hbw;
	/* 単位タイルの原点が(1,1), 左上が(0,0), 左下が(0,2) となる座標 */
	/* 上,下,左,右の順 */
	gint ox[] = {1, 1, 0, 1};  /* x軸(左右方向)の原点 */
	gint oy[] = {0, 1, 1, 1};  /* y軸(上下方向)の原点 */
	gint dx[] = {0, 0, 1, 1};  /* x軸(左右方向)の伸ばす方向 */
	gint dy[] = {1, 1, 0, 0};  /* y軸(上下方向)の伸ばす方向 */
	print_enter();
	ar = iround(gui->atomradius*gui->zoom);
	ad = ar + ar;
	hbw = iround(gui->bondwidth/2*gui->zoom);
	for (i = 0; i < 4; ++i) {
		for (j = 0; j < 2; ++j) {
			GdkGC *gc;
			GdkBitmap *mask;
			gint tilesize = gui->tilesize;
			gint halfsize = tilesize / 2;
			gint x, y, h, w;
			x = ox[i] * halfsize;
			y = oy[i] * halfsize;
			if (dx[i] == 0) {
				x -= hbw;
				w = hbw + hbw;
				h = dy[i] * halfsize + 1;  /* 隙間ができるのを避けるため+1する*/
			} else {
				y -= hbw;
				w = dx[i] * halfsize + 1;  /* 隙間ができるのを避けるため+1する*/
				h = hbw + hbw;
			}
			if (gui->mask[i][j] != NULL) {
				g_object_unref(gui->mask[i][j]);
			}
			gui->mask[i][j] = mask = gdk_pixmap_new(gui->main->window,
					tilesize, tilesize, 1);
			gc = gdk_gc_new(mask);
			/* まず全空白塗り */
			gdk_gc_set_function(gc, GDK_CLEAR);
			gdk_draw_rectangle(mask, gc, TRUE, 0, 0, tilesize, tilesize);
			/* 結合の長方形を塗る */
			gdk_gc_set_function(gc, GDK_SET);
			if (j == BOND_LINE) {
				if (i == BOND_TOP || i == BOND_BOTTOM) {
					gdk_draw_line(mask, gc, x  , y, x  , y+h);
					gdk_draw_line(mask, gc, x+w, y, x+w, y+h);
				} else {
					gdk_draw_line(mask, gc, x, y  , x+w, y  );
					gdk_draw_line(mask, gc, x, y+h, x+w, y+h);
				}
			} else {
				/* BOND_SOLID */
				gdk_draw_rectangle(mask, gc, TRUE, x, y, w, h);
			}
			/* 原子の塗りつぶしをくりぬく */
			gdk_gc_set_function(gc, GDK_CLEAR);
			gdk_draw_arc(mask, gc, TRUE,
					halfsize-ar, halfsize-ar, ad, ad, 64*0, 64*360);
			/* 原子の輪郭をくりぬく (線の太さがあるので半径が微妙に大きい) */
			gdk_gc_set_function(gc, GDK_CLEAR);
			gdk_draw_arc(mask, gc, FALSE,
					halfsize-ar, halfsize-ar, ad, ad, 64*0, 64*360);
			g_object_unref(gc);
		}
	}
	print_leave();
	return;
}
